@@ -1,4 +1,3 @@
-import atexit
 import os
 import subprocess
 import sys
@@ -9,22 +8,34 @@ from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusQAxWidgetMixin imp
 from koapy.utils.ctypes import is_admin
 from koapy.utils.logging.Logging import Logging
 from koapy.utils.platform import is_32bit
-from koapy.utils.subprocess import function_to_subprocess_args
+from koapy.utils.subprocess import Popen, function_to_subprocess_args
 
 
 class KiwoomOpenApiPlusVersionUpdater(Logging):
     def __init__(self, credentials):
         self._credentials = credentials
 
-    def disable_autologin(self):
-        self.logger.info("Disabling auto login")
+    def get_api_module_path(self):
         from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusTypeLib import (
             API_MODULE_PATH,
         )
 
         module_path = API_MODULE_PATH
-        autologin_dat = os.path.join(module_path, "system", "Autologin.dat")
-        if os.path.exists(autologin_dat):
+        return module_path
+
+    def get_autologin_dat(self):
+        module_path = self.get_api_module_path()
+        autologin_dat = module_path / "system" / "Autologin.dat"
+        return autologin_dat
+
+    def is_autologin_enabled(self):
+        autologin_dat = self.get_autologin_dat()
+        return autologin_dat.exists()
+
+    def disable_autologin(self):
+        self.logger.info("Disabling auto login")
+        autologin_dat = self.get_autologin_dat()
+        if autologin_dat.exists():
             self.logger.info("Removing %s", autologin_dat)
             os.remove(autologin_dat)
             self.logger.info("Disabled auto login")
@@ -62,9 +73,9 @@ class KiwoomOpenApiPlusVersionUpdater(Logging):
         # also it's hard to use both PySide2/PyQt5 and pywinauto at the same time in the same process due to compatibility issue.
         #
         # in order to avoid the issues mentioned above we are creating login window in a separate process.
-        # and then will do login using pywinauto in another process using `__login_using_pywinauto()` function.
+        # and then will do login using pywinauto in another process using `login_using_pywinauto()` function.
         #
-        # this process will stay live until `OnEventConnect` event happens, hopefully a successful login.
+        # this process will stay alive until `OnEventConnect` event happens, hopefully a successful login.
         #
         # for more information about the compatibility issue between PySide2/PyQt5 and pywinauto, check the following link:
         #   https://github.com/pywinauto/pywinauto/issues/472
@@ -79,8 +90,7 @@ class KiwoomOpenApiPlusVersionUpdater(Logging):
 
         cmd = function_to_subprocess_args(main)
         creationflags = subprocess.CREATE_NEW_PROCESS_GROUP
-        proc = subprocess.Popen(cmd, creationflags=creationflags)
-        atexit.register(proc.kill)
+        proc = Popen(cmd, creationflags=creationflags)
         return proc
 
     @classmethod
@@ -109,7 +119,7 @@ class KiwoomOpenApiPlusVersionUpdater(Logging):
         return app.exec_()
 
     def show_account_window(self):
-        # this function does pretty much the same job like the `__open_login_window()` function.
+        # this function does pretty much the same job like the `open_login_window()` function.
         # but the difference is that it will show up the account setting window after successful login,
         # so that we can (re-)enable the auto login functionality provided by the OpenAPI itself.
         #
@@ -129,60 +139,9 @@ class KiwoomOpenApiPlusVersionUpdater(Logging):
 
         cmd = function_to_subprocess_args(main)
         creationflags = subprocess.CREATE_NEW_PROCESS_GROUP
-        proc = subprocess.Popen(cmd, creationflags=creationflags)
-        atexit.register(proc.kill)
+        proc = Popen(cmd, creationflags=creationflags)
         return proc
 
-    @classmethod
-    def enable_autologin_using_pywinauto(cls, account_passwords):
-        import pywinauto
-
-        is_in_development = False
-
-        desktop = pywinauto.Desktop(allow_magic_lookup=False)
-        account_window = desktop.window(title_re=r"계좌비밀번호 입력 \(버전: [0-9]+.+[0-9]+\)")
-
-        try:
-            cls.logger.info("Waiting for account window to show up")
-            timeout_account_window_ready = 15
-            account_window.wait("ready", timeout_account_window_ready)
-        except pywinauto.timings.TimeoutError:
-            cls.logger.info("Cannot find account window")
-            raise
-        else:
-            cls.logger.info("Account window found")
-            if is_in_development:
-                account_window.logging.info_control_identifiers()
-
-            cls.logger.info("Enabling auto login")
-            account_window["CheckBox"].check()
-
-            account_combo = account_window["ComboBox"]
-            account_cnt = account_combo.item_count()
-
-            cls.logger.info("Putting account passwords")
-            for i in range(account_cnt):
-                account_combo.select(i)
-                account_no = account_combo.selected_text().split()[0]
-                if account_no in account_passwords:
-                    account_window["Edit"].set_text(account_passwords[account_no])
-                elif "0000000000" in account_passwords:
-                    account_window["Edit"].set_text(account_passwords["0000000000"])
-                account_window["등록"].click()
-
-            cls.logger.info("Closing account window")
-            account_window["닫기"].click()
-
-            try:
-                cls.logger.info("Waiting account window to be closed")
-                timeout_account_window_done = 5
-                account_window.wait_not("visible", timeout_account_window_done)
-            except pywinauto.timings.TimeoutError as e:
-                cls.logger.info("Cannot sure account window is closed")
-                raise RuntimeError("Cannot sure account window is closed") from e
-            else:
-                cls.logger.info("Account window closed")
-
     @classmethod
     def check_apply_simulation_window(cls):
         import pywinauto
@@ -191,7 +150,7 @@ class KiwoomOpenApiPlusVersionUpdater(Logging):
         apply_simulation_window = desktop.window(title="모의투자 참가신청")
 
         try:
-            timeout_apply_simulation = 10
+            timeout_apply_simulation = 5
             apply_simulation_window.wait("ready", timeout_apply_simulation)
         except pywinauto.timings.TimeoutError:
             pass
@@ -205,6 +164,14 @@ class KiwoomOpenApiPlusVersionUpdater(Logging):
         KiwoomOpenApiPlusQAxWidgetMixin.LoginUsingPywinauto_Impl(credentials)
         cls.check_apply_simulation_window()
 
+    @classmethod
+    def enable_autologin_using_pywinauto(cls, credentials):
+        # reusing the implementation in the mixin
+        account_passwords = credentials.get("account_passwords")
+        KiwoomOpenApiPlusQAxWidgetMixin.EnableAutoLoginUsingPywinauto_Impl(
+            account_passwords
+        )
+
     def enable_autologin(self):
         self.logger.info("Start enabling auto login")
 
@@ -212,108 +179,41 @@ class KiwoomOpenApiPlusVersionUpdater(Logging):
         account_window_proc = self.show_account_window()
 
         credentials = self._credentials
-        account_passwords = credentials.get("account_passwords")
 
         self.login_using_pywinauto(credentials)
-        self.enable_autologin_using_pywinauto(account_passwords)
+        self.enable_autologin_using_pywinauto(credentials)
+
+    @classmethod
+    def handle_version_upgrade_using_pywinauto(cls, pid):
+        return KiwoomOpenApiPlusQAxWidgetMixin.HandleVersionUpgradeUsingPywinauto_Impl(
+            pid
+        )
 
     def try_version_update_using_pywinauto(self):
-        import pywinauto
-
         self.logger.info("Trying version update")
-        self.disable_autologin()
+
+        is_autologin_enabled = self.is_autologin_enabled()
+
+        if is_autologin_enabled:
+            self.disable_autologin()
 
         login_window_proc = self.open_login_window()
 
-        desktop = pywinauto.Desktop(allow_magic_lookup=False)
-        login_window = desktop.window(title="Open API Login")
-
         credentials = self._credentials
+
         self.login_using_pywinauto(credentials)
 
-        timeout_login_successful = 4
-        timeout_version_check = 1
-        timeout_per_trial = timeout_login_successful + timeout_version_check
+        is_updated = self.handle_version_upgrade_using_pywinauto(login_window_proc.pid)
 
-        trial_timeout = 180
-        trial_count = trial_timeout / timeout_per_trial
-        while trial_count > 0:
-            try:
-                self.logger.info(
-                    "Login in progress ... timeout after %d sec",
-                    trial_count * timeout_per_trial,
-                )
-                trial_count -= 1
-                login_window.wait_not("exists", timeout_login_successful)
-                if login_window.exists():
-                    # make sure that login_window control does not exist.
-                    continue
-            except pywinauto.timings.TimeoutError as e:
-                version_window = desktop.window(title="opstarter")
-                try:
-                    version_window.wait("ready", timeout_version_check)
-                except pywinauto.timings.TimeoutError:
-                    continue
-                else:
-                    self.logger.info("Version update required")
-
-                    self.logger.info("Closing login app")
-                    login_window_proc.kill()
-                    login_window_proc.wait()
-                    self.logger.info("Killed login app process")
-                    timeout_login_screen_closed = 30
-                    login_window.close(timeout_login_screen_closed)
-                    try:
-                        login_window.wait_not("visible", timeout_login_screen_closed)
-                    except pywinauto.timings.TimeoutError as e:
-                        self.logger.info("Cannot close login window")
-                        raise RuntimeError("Cannot close login window") from e
-                    else:
-                        self.logger.info("Closed login window")
-
-                        self.logger.info("Starting to update version")
-                        version_window["Button"].click()
-
-                        versionup_window = desktop.window(title="opversionup")
-                        confirm_window = desktop.window(title="업그레이드 확인")
-
-                        try:
-                            self.logger.info("Waiting for possible failure")
-                            timeout_confirm_update = 10
-                            versionup_window.wait("ready", timeout_confirm_update)
-                        except pywinauto.timings.TimeoutError:
-                            self.logger.info("Cannot find failure confirmation popup")
-                        else:
-                            self.logger.info("Failed to update")
-                            raise RuntimeError("Failed to update") from e
-
-                        try:
-                            self.logger.info(
-                                "Waiting for confirmation popup after update"
-                            )
-                            timeout_confirm_update = 10
-                            confirm_window.wait("ready", timeout_confirm_update)
-                        except pywinauto.timings.TimeoutError as e:
-                            self.logger.info("Cannot find confirmation popup")
-                            raise RuntimeError("Cannot find confirmation popup") from e
-                        else:
-                            self.logger.info("Confirming update")
-                            confirm_window["Button"].click()
-
-                        self.logger.info("Done update")
-                        self.logger.info("Enabling auto login back")
-                        self.enable_autologin()
-                        self.logger.info("Done update, enabled auto login")
-
-                        return True
+        if is_autologin_enabled:
+            self.logger.info("Enabling auto login back")
+            self.enable_autologin()
+            if is_updated:
+                self.logger.info("Done update, enabled auto login")
             else:
-                self.logger.info("Login ended successfully")
-                self.logger.info("No version update required")
-                self.logger.info("Enabling auto login back")
-                self.enable_autologin()
                 self.logger.info("There was no version update, enabled auto login")
-                return False
-        return False
+
+        return is_updated
 
     def update_version_if_necessary(self):
         assert (
